@@ -35,6 +35,7 @@
 
 class KProcess;
 class KPtyProcess;
+class QTemporaryDir;
 
 namespace Kerfuffle
 {
@@ -351,6 +352,8 @@ private:
      */
     void writeToProcess(const QByteArray& data);
 
+    bool moveToFinalDest(const QVariantList &files, const QString &finalDest, const QTemporaryDir &tmpDir);
+
     QByteArray m_stdOutData;
     QRegExp m_existsPattern;
     QRegExp m_passwordPromptPattern;
