@@ -25,9 +25,6 @@
 
 #include "jsonarchiveinterface.h"
 
-#include <kdebug.h>
-#include <qjson/parser.h>
-
 #include <qfile.h>
 
 JSONArchiveInterface::JSONArchiveInterface(QObject *parent, const QVariantList& args)
@@ -106,4 +103,4 @@ bool JSONArchiveInterface::deleteFiles(const QList<QVariant>& files)
     return true;
 }
 
-#include "jsonarchiveinterface.moc"
+
