@@ -543,8 +543,12 @@ void Part::slotLoadingFinished(KJob *job)
     }
 
     m_view->sortByColumn(0, Qt::AscendingOrder);
-    m_view->expandToDepth(0);
 
+    // #303708: expand the first level only when there is just one root folder.
+    // Typical use case: an archive with source files.
+    if (m_view->model()->rowCount() == 1) {
+        m_view->expandToDepth(0);
+    }
 
     // After loading all files, resize the columns to fit all fields
     m_view->header()->resizeSections(QHeaderView::ResizeToContents);
