@@ -64,14 +64,15 @@ ParameterList CliPlugin::parameterList() const
         p[PreservePathSwitch] = QStringList() << QLatin1String( "x" ) << QLatin1String( "e" );
         p[PasswordSwitch] = QStringList() << QLatin1String( "-p$Password" );
         p[EncryptHeaderSwitch] = QStringList() << QLatin1String("-mhe=$Enabled");
-        p[FileExistsExpression] = QLatin1String( "already exists. Overwrite with" );
         p[WrongPasswordPatterns] = QStringList() << QLatin1String( "Wrong password" );
         p[AddArgs] = QStringList() << QLatin1String( "a" ) << QLatin1String( "$Archive" ) << QLatin1String("$EncryptHeaderSwitch") << QLatin1String( "$PasswordSwitch" ) << QLatin1String( "$Files" );
         p[DeleteArgs] = QStringList() << QLatin1String( "d" ) << QLatin1String( "$Archive" ) << QLatin1String( "$Files" );
 
+        p[FileExistsExpression] = QLatin1String( "^\\(Y\\)es / \\(N\\)o / \\(A\\)lways / \\(S\\)kip all / A\\(u\\)to rename all / \\(Q\\)uit\\? $" );
+        p[FileExistsFileName] = QStringList() << QLatin1String("^file \\./(.*)$");
         p[FileExistsInput] = QStringList()
                              << QLatin1String( "Y" ) //overwrite
-                             << QLatin1String( "N" )//skip
+                             << QLatin1String( "N" ) //skip
                              << QLatin1String( "A" ) //overwrite all
                              << QLatin1String( "S" ) //autoskip
                              << QLatin1String( "Q" ) //cancel
